/*
*  RemoteLogger utility
*  
*  source/client.c
*  snapshot 20040203
*  
*  (c) 2.2.2004 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#define VERSION "RLClient snapshot 20040203"


#include <stdio.h>
#include <string.h>
#ifdef __linux__
#include <unistd.h>
#else
#include <io.h>
#endif
#include "netlib.h"
#include "common.h"


#define MAXLINE 1024


void print_help(char *name);
int run_client(char *server, char *s_port, char *message);


int main(int argc, char *argv[])
{
        int count,
            n;
        char *argument = NULL,
             *server = NULL,
             *port = NULL,
             *message = NULL,
             line[MAXLINE];


        if (argc < 5){
                print_help(argv[0]);
                return -1;
        }

        count = argc - 1;

        while (count > 0){
	        if(argv[count][0] == '-'){

                        switch(argv[count][1]){
                                case 'i' :
                                        server = argument;
		                        break;
	                        case 'm' :
                                        message = argument;
		                        break;
                                case 'p' :
                                        port = argument;
		                        break;
	                }

                        argument = NULL;

	        } else
                        argument = argv[count];
	        
                count--;
        }

        if ((server == NULL) || (port == NULL)){
                print_help(argv[0]);
                return -1;
        }

        if (message == NULL)
                while ((n = read(fileno(stdin), line, MAXLINE - 1)) > 0){
                        line[n] = '\0';
                        run_client(server, port, line);
                }
        else
                run_client(server, port, message);

        return 0;
}


void print_help(char *name)
{
        printf("%s - utilitka pro vzdalene logovani\n", VERSION);
        printf("(c)2004 Jaroslav Beran - berk@volny.cz http://www.volny.cz/berk\n");
        printf("Licence GNU/GPL2 (http://www.gnu.cz)\n\n");

        printf("Pouziti:\n");
        printf("%s -i remote_ip -p remote_port [-m message]\n\n", name);

        printf("Priklad:\n");
        printf("%s -i 127.0.0.1 -p 1234 [-m \"Tuhle informaci chci poslat\"]\n", name);


        return;
}


int run_client(char *server, char *s_port, char *message)
{
        MYSOCK sockfd;
        struct sockaddr_in serv_addr;
        int port;

        if(is_num(s_port))
                sscanf(s_port, "%d", &port);
        else {
                fprintf(stderr, "run_client: Cislo portu neni platne %s\n", s_port);
                return -1;
        }

        if(server == NULL) 
                return -1;

        if (init_netlib()){
                fprintf(stderr, "run_client: Nepodarilo se inicializovat WinSock2\n");
                return -1;
        }

        memset((char *) &serv_addr, (int) 0, sizeof(serv_addr));

        serv_addr.sin_family      = AF_INET;
        serv_addr.sin_addr.s_addr = inet_addr(server);
        serv_addr.sin_port        = htons(port);

        sockfd = socket(AF_INET, SOCK_STREAM, 0);

        if (connect(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
                fprintf(stderr, "run_client: Nepodarilo se pripojit na server\n");
                close_netlib();
                return -1;
        }
/*
        if(vebrose)
                printf("posilam: %s (length: %u)\n", message, strlen(message));
*/
        send(sockfd, message, strlen(message), 0);


        closesocket(sockfd);
        close_netlib();

        return 0;
}
