/*
*  RemoteLogger utility
*  
*  source/logger.c
*  snapshot 20040223
*  
*  (c) 2.2.2004 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include "logger.h"


#define MAXLOGLINE 512


char *logfile = NULL;


void set_log_file(char *s)
{
        logfile = s;
        return;
}


void log_message(int type, const char *msgfmt, ...)
{
        va_list ap;
        char logmsg[MAXLOGLINE] = "";
        char stype[10];
        FILE *f;
        time_t my_t;
        struct tm *t;


        switch (type) {

                case LM_DEBUG :
                        strncpy(stype, "DEBUG ", 9);
                        stype[9] = '\0';
                        break;
                case LM_NOTICE :
                        strncpy(stype, "NOTICE", 9);
                        stype[9] = '\0';
                        break;
                case LM_WARN :
                        strncpy(stype, "WARN  ", 9);
                        stype[9] = '\0';
                        break;
                case LM_ERROR :
                        strncpy(stype, "ERROR ", 9);
                        stype[9] = '\0';
                        break;
                case LM_FATAL :
                        strncpy(stype, "FATAL ", 9);
                        stype[9] = '\0';
                        break;
                default :
                        strncpy(stype, "?     ", 9);
                        stype[9] = '\0';
        }

        va_start(ap, msgfmt);
        vsnprintf(logmsg, MAXLOGLINE, msgfmt, ap);
        va_end(ap);

        if (logfile == NULL)
                f = stdout;
        else
                if ((f = fopen(logfile, "a")) == NULL){
                        fprintf(stderr, "log_message: can't open %s\n", logfile);
                        return;
                }

        my_t = time(NULL);
        t = localtime(&my_t);
        fprintf(f, "%02d.%02d.%d %02d:%02d:%02d %s %s\n", t->tm_mday, \
                t->tm_mon + 1, t->tm_year + 1900, t->tm_hour, \
                t->tm_min, t->tm_sec, stype, logmsg);

        if (f != stdout)
                fclose(f);

        return;
}
