/*
*  RemoteLogger utility
*  
*  source/server.c
*  snapshot 20040224
*  
*  (c) 2.2.2004 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#define VERSION "RLServer snapshot 20040224"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef __linux__
#include <signal.h>
#endif
#include "netlib.h"
#include "logger.h"
#include "common.h"


#define MAXLINE 1024


FILE *f = NULL;


void print_help(char *name);
int run_server(char *s_port, char *logfile, int flsh);
#ifdef __linux__
void sighup_handler(int i);
void sigterm_handler(int i);
#endif


int main(int argc, char *argv[])
{
        int count,
            flsh = 0,
            ret;
        char *argument = NULL,
             *port = NULL,
             *logfile = NULL;


        if (argc == 1) {
                print_help(argv[0]);
                return -1;
        }

        count = argc - 1;

        while (count > 0){
	        if(argv[count][0] == '-'){

                        switch(argv[count][1]){
                                case 'e' :
                                        set_log_file(argument);
		                        break;
                                case 'f' :
                                        flsh = 1;
		                        break;
                                case 'l' :
                                        logfile = argument;
		                        break;
                                case 'p' :
                                        port = argument;
		                        break;
	                }

                        argument = NULL;

	        } else
                        argument = argv[count];
	        
                count--;
        }

        if ((port == NULL) || (logfile == NULL)){
                print_help(argv[0]);
                return -1;
        }

#ifdef __linux__
        if(signal(SIGHUP, sighup_handler) == SIG_ERR){
                log_message(LM_ERROR, "main: Can't catch SIGHUP");
                return -1;
        }

        if(signal(SIGTERM, sigterm_handler) == SIG_ERR){
                log_message(LM_ERROR, "main: Can't catch SIGTERM");
                return -1;
        }
#endif

        if ((f = fopen(logfile, "a")) == NULL){
                log_message(LM_ERROR, "run_server: Nemohu otevrit soubor %s", logfile);
                return -1;
        }

        ret = run_server(port, logfile, flsh);
        fclose(f);

        return ret;
}


void print_help(char *name)
{
        printf("%s - utilitka pro vzdalene logovani\n", VERSION);
        printf("(c)2004 Jaroslav Beran - berk@volny.cz http://www.volny.cz/berk\n");
        printf("Licence GNU/GPL2 (http://www.gnu.cz)\n\n");

        printf("Pouziti:\n");
        printf("%s -p port -l logfile [-f] [-e error.log]\n\n", name);

        printf("Priklad:\n");
        printf("%s -p 1234 -l myfile.log\n", name);

        printf("\nPoznamka:\n");
        printf("-f\tRika ze se prichozi data maji okamzite zapisovat do souboru\n");
        #ifdef __linux__        
        printf("SIG_HUP zavola fflush a ulozi obsah souborove kese\n");
        #endif

        return;
}


int run_server(char *s_port, char *logfile, int flsh)
{
        int clilen, n;
        MYSOCK sockfd, newsockfd;
        struct sockaddr_in cli_addr, serv_addr;
        char radek[MAXLINE];
        int port;


        if(is_num(s_port))
                sscanf(s_port, "%d", &port);
        else {
                log_message(LM_ERROR, "run_server: Cislo portu neni platne %s", s_port);
                return -1;
        }

        my_touch(logfile);

        printf("Posloucham na portu %d\n", port);
        printf("Loguju do %s\n", logfile);

#ifdef __linux__
        deamonize();
#endif
        if (init_netlib()){
                log_message(LM_ERROR, "run_server: Nepodarilo se inicializovat WinSock2");
                return -1;
        }

        sockfd = socket(AF_INET, SOCK_STREAM, 0);

        memset((char *) &serv_addr, (int) 0, sizeof(serv_addr));
        
        serv_addr.sin_family      = AF_INET;
        serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
        serv_addr.sin_port        = htons(port);

        if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0){
                log_message(LM_ERROR, "run_server: Can't bind local address");
                close_netlib();
                return -1;
        }

        listen(sockfd, 5);

        for ( ; ; ) {

                clilen = sizeof(cli_addr);
                newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);

                n = 1;
                while (n != 0) {

                        radek[0] = '\0';
                        n = tcp_readline(newsockfd, radek, MAXLINE);

                        if (n < 0) {
                                log_message(LM_ERROR, "run_server: Chyba pri cteni radky (%d)", n);
                                n = 0;
                        }

                        if (strlen(radek) > 0) {
                                fprintf(f, "%s", radek);
                                if (flsh) fflush(f);
                        }
                }

                closesocket(newsockfd);
        }

        closesocket(sockfd);
        close_netlib();

        return 0;
}


#ifdef __linux__
void sighup_handler(int i)
{
        if (f != NULL)
                fflush(f);

        return;
}


void sigterm_handler(int i)
{
        if (f != NULL)
                fclose(f);

        exit(0);
}
#endif
